/*
    FreeRTOS V6.1.1 - Copyright (C) 2011 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"

/* Demo includes. */
#include "basic_io.h"


#include "printf.h"

#include <stdint.h>

/* Used as a loop counter to create a very crude delay. */
#define mainDELAY_LOOP_COUNT		( 0xfffff )

/* The task functions. */
void vTask1( void *pvParameters );
void vTask2( void *pvParameters );
void vTask3( void *pvParameters );
void vTask4( void *pvParameters );

/*-----------------------------------------------------------*/

int main( void )
{
    const char* welcomeMsg = "UART3 Online:\r\n";
    //SystemInit();	//Called by startup code
    Log_start(UART3,9600);
    /* Init the semi-hosting. */
    printf( "\n" );
    /* Create one of the two tasks. */
    xTaskCreate(	vTask1,		/* Pointer to the function that implements the task. */
                    "Task 1",	/* Text name for the task.  This is to facilitate debugging only. */
                    240,		/* Stack depth in words. */
                    NULL,		/* We are not using the task parameter. */
                    1,			/* This task will run at priority 1. */
                    NULL );		/* We are not using the task handle. */
    /* Create the other task in exactly the same way. */
    xTaskCreate( vTask2, "Task 2", 240, NULL, 1, NULL );
    xTaskCreate( vTask3, "Task 3", 240, NULL, 1, NULL );
    xTaskCreate( vTask4, "Task 4", 240, NULL, 1, NULL );
    /* Start the scheduler so our tasks start executing. */
    vTaskStartScheduler();
    /* If all is well we will never reach here as the scheduler will now be
    running.  If we do reach here then it is likely that there was insufficient
    heap available for the idle task to be created. */
    for( ;; );
    return 0;
}
/*-----------------------------------------------------------*/



/*
 * This  prints through UART3 his task name and current tick counter
 * */
void vTask1( void *pvParameters )
{
    const char *pcTaskName = "Task 1 is running\n";
    volatile unsigned long ul;
    static uint32_t counter = 0;
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        counter = xTaskGetTickCount();
        /* Send through UART3 pcTaskName string */
        PrintF_print(UART3, "%s %d", pcTaskName, counter);
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/
/**
 * This task uses Log_log to log a text
 */
void vTask2( void *pvParameters )
{
    uint32_t counter = 0;
    const char *pcTaskName = "Task 2 is running\n";
    volatile unsigned long ul;
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        Log_log(pcTaskName);
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/

/*
 * This  prints through UART3 his task name and current tick counter
 * */
void vTask3( void *pvParameters )
{
    uint32_t counter = 0;
    const char *pcTaskName = "Task 3 is running\n";
    volatile unsigned long ul;
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        /* Send through UART3 pcTaskName string */
        PrintF_print(UART3, "%s %d", pcTaskName, counter);
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/

/*
 * This  prints through UART3 his task name and current tick counter
 * */
void vTask4( void *pvParameters )
{
    uint32_t counter = 0;
    uint8_t buffer[64];
    const char *pcTaskName = "Task 4 is running\n";
    volatile unsigned long ul;
    /* As per most tasks, this task is implemented in an infinite loop. */
    for( ;; )
    {
        /* Print out the name of this task. */
        vPrintString( pcTaskName );
        int32_t ret = PrintF_scan(UART3, buffer, 4000);
        if( ret > 0 )
        {
            /* Send through UART3 pcTaskName string */
            PrintF_print(UART3, "Readed %s", buffer);
        }
        else
        {
            PrintF_print(UART3, "Error reading %d",ret);
        }
        /* Delay for a period. */
        for( ul = 0; ul < mainDELAY_LOOP_COUNT; ul++ )
        {
            /* This loop is just a very crude delay implementation.  There is
            nothing to do in here.  Later exercises will replace this crude
            loop with a proper delay/sleep function. */
        }
    }
}
/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
    /* This function will only be called if an API call to create a task, queue
    or semaphore fails because there is too little heap RAM remaining. */
    for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
    /* This function will only be called if a task overflows its stack.  Note
    that stack overflow checking does slow down the context switch
    implementation. */
    for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
    /* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
    /* This example does not use the tick hook to perform any processing. */
}


