/*
===============================================================================
 Name        : ecotest2.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "LPC17xx.h"
#include "uart.h"


#include <stdio.h>








/* Own libs */
#include "led.h"
#include "printf.h"
#include "uart.h"
#include "hc05.h"
#include <vgear.h>
#include <vgear_request.h>
#include <vgear_command.h>
#include <adc.h>
#include <ret.h>
#include <timer.h>

#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include  "bluegearconstants.h"
#include "watchdog.h"






#define HC05PowerPin 						( 22 )
#define HC05KeyPin 						    ( 23 )




static void taskSampleADC( void *taskParameters );
static void taskVGEARRequester( void *taskParameters );
static void taskBluetoothListener( void *taskParameters );








static char commandBuffer[BLUETOOTH_COMMAND_BUFFER_SIZE];
static char outputBuffer[BLUETOOTH_OUTPUT_BUFFER_SIZE];
static uint32_t indexCommandBuffer = 0;
static ECU_STATUS ecuStatus;


/*-----------------------------------------------------------*/



int main(void)
{
    printf("Previous()\n");
    Watchdog_init(60000);
    Watchdog_pulse();
    xTaskCreate( taskBluetoothListener , NULL , configMINIMAL_STACK_SIZE * 6, NULL, BLUETOOTH_TASK_PRIORITY , NULL );
    Watchdog_pulse();
    xTaskCreate( taskSampleADC , ( signed char * ) "Rx" , configMINIMAL_STACK_SIZE * 2, NULL, ADC_SAMPLE_TASK_PRIORITY , NULL );
    Watchdog_pulse();
    xTaskCreate( taskVGEARRequester , NULL , configMINIMAL_STACK_SIZE * 4, NULL, VGEAR_TASK_PRIORITY , NULL );
    Watchdog_pulse();
#ifdef _USE_DEBUG_MESSAGES
    printf("Created all the task!\n");
#endif
    /* Start the tasks running. */
    vTaskStartScheduler();
#ifdef _USE_DEBUG_MESSAGES
    printf("start()!\n");
#endif
#ifdef _USE_DEBUG_MESSAGES
    printf("Error allocating space: HEAP maybe?\n");
#endif
    for( ;; );
}

static void taskBluetoothListener( void *taskParameters )
{
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
    int32_t ret ;
    char readBuffer[BLUETOOTH_READ_BUFFER_SIZE];
    portTickType xNextWakeTime;
    xNextWakeTime = xTaskGetTickCount();
    ret = 	HC05_start(BLUETOOTH_UART, BLUETOOTH_BAUDRATE, BLUETOOTH_POWER_PIN, BLUETOOTH_KEY_PIN);
    Timer_sleepMs(TIMER0,BLUETOOTH_WAIT_COMMAND_MODE_MS);
    memset(outputBuffer,'\0',BLUETOOTH_OUTPUT_BUFFER_SIZE);
    Watchdog_pulse();
    Watchdog_pulse();
    HC05_closeCommandMode(BLUETOOTH_UART);
    Watchdog_pulse();
    //uint32_t k = 0;
    Timer_sleepMs(TIMER0,BLUETOOTH_WAIT_COMMAND_MODE_MS);
    while(1)
    {
        Watchdog_pulse();
        //k++;
        memset(readBuffer,'\0',BLUETOOTH_READ_BUFFER_SIZE);
        //Read command
        int32_t readSize =   PrintF_scan(BLUETOOTH_UART,readBuffer,BLUETOOTH_WAIT_READ_MS);
#ifdef _USE_DEBUG_MESSAGES
        printf("%s i: %d buffer: %s\n",__PRETTY_FUNCTION__,ret,readBuffer);
#endif
        PrintF_print(BLUETOOTH_UART,outputBuffer);
        int32_t ret;
        for(int32_t i = 0; i < readSize ; i++)
        {
            commandBuffer[indexCommandBuffer] = readBuffer[i];
            if( readBuffer[i] == '\n')
            {
                commandBuffer[indexCommandBuffer] = '\0';
#ifdef _USE_DEBUG_MESSAGES
                printf("CommandBuffer '%s'\n",commandBuffer);
#endif
                //Proccess commandBuffer
                ret = VGEAR_parseCommand(&ecuStatus, commandBuffer, outputBuffer);
#ifdef _USE_DEBUG_MESSAGES
                printf("ret: %d, OutputBuffer '%s'\n",ret, outputBuffer);
#endif
                if( RET_OK == ret )
                {
                    PrintF_print(BLUETOOTH_UART,outputBuffer);
                }
                indexCommandBuffer = 0;
                memset(commandBuffer,'\0',BLUETOOTH_COMMAND_BUFFER_SIZE);
            }
            else
            {
                indexCommandBuffer++;
                if( indexCommandBuffer >= BLUETOOTH_COMMAND_BUFFER_SIZE )
                {
                    indexCommandBuffer = 0;
                }
            }
        }
        vTaskDelayUntil( &xNextWakeTime, BLUETOOTH_WAIT_TASK );
    }
}

static void taskVGEARRequester( void *taskParameters )
{
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
    int32_t ret = 	PrintF_start(VGEAR_UART, VGEAR_BAUDRATE);
    char readBuffer[VGEAR_READ_BUFER_SIZE];
    portTickType xNextWakeTime;
    xNextWakeTime = xTaskGetTickCount();
    while(ret != RET_OK )
    {
        //Try again
        ret = 	PrintF_start(VGEAR_UART, VGEAR_BAUDRATE);
        //Sleep
        vTaskDelayUntil( &xNextWakeTime, VGEAR_WAIT_RECONNECT );
#ifdef _USE_DEBUG_MESSAGES
        printf("%s Waitting...\n",__PRETTY_FUNCTION__);
#endif
    }
    while(1)
    {
        PrintF_print(VGEAR_UART, "%s\n", C_ALL );
        memset(readBuffer,'\0',VGEAR_READ_BUFER_SIZE);
        ret =   PrintF_scan(VGEAR_UART,readBuffer,VGEAR_WAIT_READ_MS);
#ifdef _USE_DEBUG_MESSAGES
        printf("%s i: %d buffer: %s\n",__PRETTY_FUNCTION__,ret,readBuffer);
#endif
        if (ret > 0 )
        {
            //Try to parse
            ret = VGEAR_stringToECUStatus(readBuffer,&ecuStatus);
        }
        vTaskDelayUntil( &xNextWakeTime, VGEAR_WAIT_TASK );
    }
}


static void taskSampleADC( void *taskParameters )
{
    Watchdog_pulse();
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
    int32_t ret;
    ADC_Init();
    portTickType xNextWakeTime;
    xNextWakeTime = xTaskGetTickCount();
    while(1)
    {
        Watchdog_pulse();
        ret = ADC_Read( ADC_ACCEL_X, &(ecuStatus.accelX) );
        if ( ret != RET_OK )
        {
            ecuStatus.accelX = 0;
            ecuStatus.accelY = 0;
            ecuStatus.accelZ = 0;
        }
        else
        {
            ADC_Read( ADC_ACCEL_Y, &(ecuStatus.accelY) );
            ADC_Read( ADC_ACCEL_Z, &(ecuStatus.accelZ) );
        }
#ifdef _USE_DEBUG_MESSAGES
        printf("ADC TASK %u %u %u\n ", ecuStatus.accelX, ecuStatus.accelY, ecuStatus.accelZ  );
#endif
        vTaskDelayUntil( &xNextWakeTime, ADC_WAIT_TASK );
    }
}






void vApplicationIdleHook( void )
{
    /* This example does not use the idle hook to perform any processing. */
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
    /* This example does not use the tick hook to perform any processing. */
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
}

void vApplicationMallocFailedHook( void )
{
    /* This function will only be called if an API call to create a task, queue
    or semaphore fails because there is too little heap RAM remaining. */
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
    for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
#ifdef _USE_DEBUG_MESSAGES
    printf("%s\n",__PRETTY_FUNCTION__);
#endif
    /* This function will only be called if a task overflows its stack.  Note
    that stack overflow checking does slow down the context switch
    implementation. */
    for( ;; );
}




