/*
 * hc05.h
 *
 *  Created on: 31/3/2015
 *      Author: yango
 */



#include <stdint.h>
#include <stdarg.h>
#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "semphr.h"
#include "type.h"

#ifndef HC05_H_
#define HC05_H_



static const uint32_t HC05_WAIT_POWEROFF_MS = 100;

/* Public interface */


/** Initializes a HC05 module
 * @precondition baudrate and uartId are correct.
 *
 * @param uartId   UART number
 * @param baudrate       valid baudraute

 *
 * @return 0 in case of success, otherwise returns a negative value defined in ret.h
 */
int32_t HC05_start(UART_ID uartId, uint32_t baudRate, uint8_t powerPin , uint8_t keyPin);

/** Ends command mode
 * @precondition uartId is correct and HC05 is in command mode
 *
 * @param uartId   UART number
 *
 * @return 0 in case of success, otherwise returns a negative value defined in ret.h
 */
int32_t HC05_closeCommandMode(UART_ID uartId);

/** Changes PIN of HC05. Default is 1234
 * @precondition uartId is correct and HC05 is in command mode
 *
 * @param uartId   UART number
 * @param *strPIN string of new PIN
 *
 * @return 0 in case of success, otherwise returns a negative value defined in ret.h
 */
int32_t HC05_changePIN(UART_ID uartId, const char *strPIN);

/** Changes Bluetooth name of HC05. Default is HC05
 * @precondition uartId is correct and HC05 is in command mode
 *
 * @param uartId   UART number
 * @param *strName string of new name
 *
 * @return 0 in case of success, otherwise returns a negative value defined in ret.h
 */
int32_t HC05_changeName(UART_ID uartId, const char *strName);



#endif /* HC05_H_ */
