/*
 * hc05.c
 *
 *  Created on: 31/3/2015
 *      Author: yango
 */
#include <stdint.h>
#include <stdio.h>

#include "hc05.h"
#include "printf.h"
#include "uart.h"
#include "ret.h"
#include "timer.h"

static int32_t isInitialized[UART_ID_LENGTH] =
{ 0, 0, 0, 0 }; /* array to store if a UART is initialized */

static xSemaphoreHandle lstMutex[UART_ID_LENGTH]; /* array of mutex */

static uint32_t lstBaudRates[UART_ID_LENGTH] =
{ 9600, 9600, 9600, 9600 };

static uint32_t lstKeyPin[UART_ID_LENGTH] =
{ 0, 0, 0, 0 };
static uint32_t lstPowerPin[UART_ID_LENGTH] =
{ 0, 0, 0, 0 };

/* Private methods */
int32_t _sendCommand(UART_ID uartId, const char* command);

//PRE: powerPin y keyPin ya están puestos como GPIO
int32_t HC05_start(UART_ID uartId, uint32_t baudRate, uint8_t powerPin,
                   uint8_t keyPin)
{
    //baudRate = 38400;
    int32_t ret = PrintF_start(uartId, 38400);
    if (RET_OK == ret)
    {
        isInitialized[uartId] = 1;
        lstBaudRates[uartId] = baudRate;
        lstMutex[uartId] = xSemaphoreCreateMutex();
        lstPowerPin[uartId] = powerPin;
        lstKeyPin[uartId] = keyPin;
        /* Clear bits 12 and 11 on PINSEL1 */
        //LPC_PINCON->PINSEL1	&= ( ~( 3 << 12 ) );
        /* Data direction 1 -> OUT */
        LPC_GPIO0->FIODIR |= (1 << keyPin);
        //LPC_PINCON->PINSEL1	&= ( ~( 3 << 12 ) );
        /* Data direction 1 -> OUT */
        LPC_GPIO0->FIODIR |= (1 << powerPin);
        //Set HIGH
        LPC_GPIO0->FIOSET = 1 << keyPin;
        printf("power off\n");
        //Key to high
        LPC_GPIO0->FIOSET = 1 << keyPin;
        //Set power off
        LPC_GPIO0->FIOSET = 1 << powerPin;
        //
        Timer_sleepMs(TIMER0, HC05_WAIT_POWEROFF_MS);
        printf("power on\n");
        //Set power on
        LPC_GPIO0->FIOCLR = 1 << powerPin;
    }
    return ret;
}

int32_t HC05_closeCommandMode(UART_ID uartId)
{
    if (1 != isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if ( pdTRUE != xSemaphoreTake(lstMutex[uartId], portMAX_DELAY))
    {
        return E_CANNOT_MUTEX;
    }
    //Close command mode
    PrintF_print(uartId, "AT+INIT\r\n");
    char buffer[32];
    PrintF_scan(uartId, buffer, 300);
    printf("RES: %s\n", buffer);
    //printf("power off\n");
    //Key to low
    LPC_GPIO0->FIOCLR = 1 << lstKeyPin[uartId];
    //Set power off
    LPC_GPIO0->FIOSET = 1 << lstPowerPin[uartId];
    //
    Timer_sleepMs(TIMER0, HC05_WAIT_POWEROFF_MS);
    //printf("power on\n");
    //Set power on
    LPC_GPIO0->FIOCLR = 1 << lstPowerPin[uartId];
    Timer_sleepMs(TIMER0, 500);
    //Configure UART to desired baudRaute
    int32_t ret = PrintF_start(uartId, lstBaudRates[uartId]);
    xSemaphoreGive(lstMutex[uartId]);
    return ret;
}

int32_t _sendCommand(UART_ID uartId, const char *command)
{
    char buffer[32];
    int32_t ret = PrintF_print(uartId, "%s", command);
    if (RET_OK == ret)
    {
        //Read confirmation
        ret = PrintF_scan(uartId, buffer, 1000);
        if (ret >= 2)
        {
            //Check if device answers with OK
            if ((buffer[0] == 'O') && (buffer[1] == 'K'))
            {
                ret = RET_OK;
            }
            else
            {
                //Response not expected
                ret = E_ERROR;
            }
        }
    }
    return ret;
}

int32_t HC05_changePIN(UART_ID uartId, const char *strPIN)
{
    if (1 != isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if ( pdTRUE != xSemaphoreTake(lstMutex[uartId], portMAX_DELAY))
    {
        return E_CANNOT_MUTEX;
    }
    char buffer[16];
    sprintf(buffer, "AT+PSWD=%s\r\n", strPIN);
    int32_t ret = _sendCommand(uartId, buffer);
    xSemaphoreGive(lstMutex[uartId]);
    return ret;
}

int32_t HC05_changeName(UART_ID uartId, const char *strName)
{
    if (1 != isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if ( pdTRUE != xSemaphoreTake(lstMutex[uartId], portMAX_DELAY))
    {
        return E_CANNOT_MUTEX;
    }
    char buffer[16];
    sprintf(buffer, "AT+NAME=%s\r\n", strName);
    int32_t ret = _sendCommand(uartId, buffer);
    xSemaphoreGive(lstMutex[uartId]);
    return ret;
}

