/*
 * kwp2000.c
 *
 *  Created on: May 30, 2015
 *      Author: yango
 */
#include "kwp2000.h"
#include "ret.h"
#include "timer.h"

static int32_t isInitialized[UART_ID_LENGTH] =
{ 0, 0, 0, 0 }; /* array to store if a UART is initialized */

static xSemaphoreHandle lstMutex[UART_ID_LENGTH]; /* array of mutex */

static uint32_t lstPulsePin[UART_ID_LENGTH] =
{ 0, 0, 0, 0 };

static uint8_t REQUEST_STATUS[KWP2000_REQUEST_STATUS_SIZE] = { 0x80,0x12,0xF1,0x2,0x21,0x08,0xAE } ;


//PRE: powerPin y keyPin ya están puestos como GPIO
int32_t KWP2000_start(UART_ID uartId, uint8_t pulsePin)
{
    //baudRate = 38400;
    int32_t ret = PrintF_start(uartId, 10400);
    if (RET_OK == ret)
    {
        isInitialized[uartId] = 1;
        lstMutex[uartId] = xSemaphoreCreateMutex();
        lstPulsePin[uartId] = pulsePin;
    }
    return ret;
}

uint8_t KWP2000_ISOChecksum(uint8_t *data, uint8_t len)
{
    uint8_t crc = 0;
    uint8_t i = 0;
    for (i = 0; i < len; i++)
    {
        crc = crc + data[i];
    }
    return crc;
}

int32_t KWP2000_connect(UART_ID uartId)
{
    uint8_t bufferRead[KWP2000_MAX_DATA_RECV_SIZE];
    if (1 != isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if (pdTRUE != xSemaphoreTake(lstMutex[uartId], portMAX_DELAY))
    {
        return E_CANNOT_MUTEX;
    }
    //Pulse!
    LPC_GPIO0->FIOCLR = 1 << lstPulsePin[uartId];
    //
    Timer_sleepMs(TIMER0, KWP2000_PULSE_WAIT_MS);
    LPC_GPIO0->FIOSET = 1 << lstPulsePin[uartId];
    Timer_sleepMs(TIMER0, KWP2000_PULSE_WAIT_MS);
    uint8_t dataStream[5];
    dataStream[0] = 0x81;
    dataStream[1] = 0x12;
    dataStream[2] = 0xf1;
    dataStream[3] = 0x81;
    dataStream[4] = 0x05;
    PrintF_print(uartId, "%c", dataStream[0]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c", dataStream[1]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c", dataStream[2]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c", dataStream[3]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c", dataStream[4]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    //Wait  for response
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS * 5);
    uint8_t response[32];
    uint8_t i = 0;
    //Read first 8 bytes
    int32_t readed = PrintF_scan(uartId, bufferRead, KWP2000_RESPONSE_WAIT_MS);
    xSemaphoreGive(lstMutex[uartId]);
    if (readed > 0)
    {
        return RET_OK;
    }
    return E_ERROR;
}

int32_t KWP2000_requestData(UART_ID uartId, char * outputBuffer )
{
    if (1 != isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if (pdTRUE != xSemaphoreTake(lstMutex[uartId], portMAX_DELAY))
    {
        return E_CANNOT_MUTEX;
    }
    PrintF_print(uartId, "%c",REQUEST_STATUS[0]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c",REQUEST_STATUS[1]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c",REQUEST_STATUS[2]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c",REQUEST_STATUS[3]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c",REQUEST_STATUS[4]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c",REQUEST_STATUS[5]);
    Timer_sleepMs(TIMER0, KWP2000_BYTE_DELAY_MS);
    PrintF_print(uartId, "%c",REQUEST_STATUS[6]);
    Timer_sleepMs(TIMER0, KWP2000_RESPONSE_WAIT_MS);
    int8_t ret = PrintF_scan(uartId,outputBuffer,KWP2000_RESPONSE_WAIT_MS);
    xSemaphoreGive(lstMutex[uartId]);
    return ret;
}
