/*
 * printf.c
 *
 *  Created on: 16/3/2015
 *      Author: dzayas
 */

#include "printf.h"
#include "timer.h"
#include "uart.h"
#include "string.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include "ret.h"



//External variables
extern volatile uint32_t UART0Count;
extern volatile uint8_t UART0Buffer[BUFSIZE];

extern volatile uint32_t UART1Count;
extern volatile uint8_t UART1Buffer[BUFSIZE];

extern volatile uint32_t UART3Count;
extern volatile uint8_t UART3Buffer[BUFSIZE];


/* private variables */



static int32_t isInitialized[UART_ID_LENGTH] = { 0,0,0,0 }; /* array to store if a UART is initialized */

static xSemaphoreHandle lstMutex[ UART_ID_LENGTH ] ; /* array of mutex */

static char fmtbuf[16];


/* Private methods */

void _disableInterruptUART( UART_ID uartId );

void _enableInterruptUART( UART_ID uartId );

uint32_t _getUARTBufferSize(UART_ID uartId, uint32_t  volatile *dataSize);

void _clearUARTBuffer(UART_ID uartId);


int32_t PrintF_start(UART_ID uartId, uint32_t baudRate)
{
    int32_t ret;
    if ((uartId >= UART_ID_LENGTH) || (UART2 == uartId))
    {
        return E_INCORRECT_UART;
    }
    if (1 == isInitialized[uartId])
    {
        //Allow to reconfigure
        _disableInterruptUART(uartId);
        ret = UART_init(uartId, baudRate);
        _enableInterruptUART(uartId);
        return ret;
    }
    //initialize the correct uart
    ret = UART_init(uartId, baudRate);
    //UART_send(3, __FUNCTION__, strlen(__FUNCTION__));
    //UART_send(3, "\n", 1);
    if ( TRUE == ret)
    {
        lstMutex[uartId] = xSemaphoreCreateMutex();
        isInitialized[uartId] = 1;
        return RET_OK;
    }
    return E_ERROR;
}



int32_t PrintF_print(UART_ID uartId, const char *strFormat, ...)
{
    //UART_send(3, __FUNCTION__, strlen(__FUNCTION__));
    //UART_send(3, "\n", 1);
    const char *p;
    va_list lstArgs;
    char *s;
    int32_t i;
    //Parameter check
    if ((uartId >= UART_ID_LENGTH) || (UART2 == uartId))
    {
        return E_INCORRECT_UART;
    }
    if (0 == isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if( NULL == strFormat )
    {
        return E_NULL;
    }
    //xSemaphoreTake(lstMutex[uartId], portTICK_RATE_MS/ 50);
    if( pdTRUE !=  xSemaphoreTake(lstMutex[uartId], portMAX_DELAY) )
    {
        return E_CANNOT_MUTEX;
    }
    //va_start(lstArgs, uartId);
    va_start(lstArgs, strFormat);
    for (p = strFormat; *p != '\0'; p++)
    {
        if (*p != '%')
        {
            UART_send(uartId, p, 1);
            continue;
        }
        switch (*++p)
        {
        case 'c':
            i = va_arg(lstArgs, int);
            sprintf(fmtbuf, "%c", i);
            UART_send(uartId, fmtbuf, strlen(fmtbuf));
            break;
        case 'd':
            i = va_arg(lstArgs, int);
            memset(fmtbuf,'\0',16);
            sprintf(fmtbuf, "%d", i);
            UART_send(uartId, fmtbuf, strlen(fmtbuf));
            break;
        case 's':
            s = va_arg(lstArgs, char *);
            UART_send(uartId, s, strlen((const char *) s));
            break;
        case 'x':
            i = va_arg(lstArgs, int);
            sprintf(fmtbuf, "%x", i);
            UART_send(uartId, fmtbuf, strlen(fmtbuf));
            break;
        case '%':
            UART_send(uartId, p, 1);
            break;
        }
    }
    xSemaphoreGive(lstMutex[uartId]);
    va_end(lstArgs);
    return RET_OK;
}


int32_t PrintF_scan(UART_ID uartId, char *buffer, uint32_t delayMs )
{
    /* Parameter check  */
    if ((uartId >= UART_ID_LENGTH) || (UART2 == uartId))
    {
        return E_INCORRECT_UART;
    }
    if (0 == isInitialized[uartId])
    {
        return E_NON_INITIALIZED;
    }
    if( NULL == buffer )
    {
        return E_NULL;
    }
    /* Mutex for current UART uartId */
    //xSemaphoreTake(lstMutex[uartId], portTICK_RATE_MS/ 50);
    if ( pdTRUE != xSemaphoreTake(lstMutex[uartId], portMAX_DELAY))
    {
        return E_CANNOT_MUTEX;
    }
    volatile uint32_t dataSize;
    volatile uint8_t* ptrBuffer = NULL;
    switch( uartId )
    {
    case UART0:
        ptrBuffer = &UART0Buffer[0];
        break;
    case UART1:
        ptrBuffer = &UART1Buffer[0];
        break;
    case UART3:
        ptrBuffer = &UART3Buffer[0];
        break;
    default:
        return E_INCORRECT_UART;
        break;
    }
    ////_enableInterruptUART(uartId);
    //sleep_ms(delayMs);
    Timer_sleepMs(TIMER0, delayMs);
    int32_t i = _getUARTBufferSize(uartId, &dataSize);
    ///_disableInterruptUART(uartId);
    //A pesar de que ptrBuffer se ha declarado volatile
    //podemos estar seguros que esta región de memoria
    //no se modifica porque hemos deshabilitado las interrupciones
    //de la UART
    memcpy( buffer, (uint8_t*) ptrBuffer, i );
    if ( i > 0 )
    {
        _clearUARTBuffer(uartId);
        dataSize = 0;
        i++;
        ptrBuffer[i] = '\0';
    }
    xSemaphoreGive(lstMutex[uartId]);
    return i;
}

/** disables data recieve interrupt from UART uartId
 *
 * @param uartId   UART number
 */
void _disableInterruptUART(UART_ID uartId)
{
    switch (uartId)
    {
    case UART0:
        LPC_UART0->IER = IER_THRE | IER_RLS; /* Disable RBR */
        break;
    case UART1:
        LPC_UART1->IER = IER_THRE | IER_RLS; /* Disable RBR */
        break;
    case UART3:
        LPC_UART3->IER = IER_THRE | IER_RLS; /* Disable RBR */
        break;
    default:
        //Do nothing
        break;
    }
}


/** enables data recieve interrupt from UART uartId
 *
 * @param uartId   UART number
 */
void _enableInterruptUART(UART_ID uartId)
{
    switch (uartId)
    {
    case UART0:
        //NVIC_EnableIRQ(UART0_IRQn);
        LPC_UART0->IER = IER_THRE | IER_RLS | IER_RBR; /* Re-enable RBR */
        break;
    case UART1:
        //NVIC_EnableIRQ(UART1_IRQn);
        LPC_UART1->IER = IER_THRE | IER_RLS | IER_RBR; /* Re-enable RBR */
        break;
    case UART3:
        //NVIC_EnableIRQ(UART3_IRQn);
        LPC_UART3->IER = IER_THRE | IER_RLS | IER_RBR; /* Re-enable RBR */
        break;
    default:
        //Do nothing
        break;
    }
}


/** Clears buffers from UART uartId
 *
 * @param uartId   UART number
 */
void _clearUARTBuffer(UART_ID uartId)
{
    switch (uartId)
    {
    case UART0:
        UART0Count = 0;
        memset(UART0Buffer ,'\0',BUFSIZE);
        break;
    case UART1:
        UART1Count = 0;
        memset(UART1Buffer ,'\0',BUFSIZE);
        break;
    case UART3:
        UART3Count = 0;
        memset(UART3Buffer ,'\0',BUFSIZE);
        break;
    default:
        //Do nothing
        break;
    }
}


/** Returns a pointer to a UART uartId buffer size
 *
 * @param uartId   UART number
 * @param *dataSize   pointer to a valid uint32_t
 */
uint32_t _getUARTBufferSize(UART_ID uartId, uint32_t  volatile *dataSize)
{
    switch (uartId)
    {
    case UART0:
        dataSize = &UART0Count;
        return UART0Count;
        break;
    case UART1:
        dataSize = &UART1Count;
        return UART1Count;
        break;
    case UART3:
        dataSize = &UART3Count;
        return UART3Count;
        break;
    default:
        dataSize = NULL;
        //Do nothing
        break;
    }
    return 0;
}





