/*
 * vgear_command.c
 *
 *  Created on: Apr 18, 2015
 *      Author: yango
 */

#include "vgear.h"
#include "type.h"
#include "ret.h"

#include <stddef.h>
#include <string.h>
#include <stdio.h>

void removeNewLine(char* buffer)
{
    while (*buffer != '\0')
    {
        if (*buffer == '\n')
        {
            *buffer = '\0';
            return;
        }
        buffer++;
    }
}

int8_t VGEAR_answerThrottle(ECU_STATUS* ecuStatus,  char* response)
{
    sprintf(response, "%u\n", ecuStatus->currentThrottle);
    return RET_OK;
}

int8_t VGEAR_answerRPM(ECU_STATUS* ecuStatus, char* response)
{
    sprintf(response, "%u\n", ecuStatus->currentRPM);
    return RET_OK;
}

int8_t VGEAR_answerAirTemperature(ECU_STATUS* ecuStatus,char* response)
{
#ifdef NO_FLOAT_SPRINTF
    char tmpBuffer[12];
    dtostrf(ecuStatus->tempAir, 3, 2, tmpBuffer);
    sprintf(response, "%s\n", tmpBuffer);
#else
    sprintf(response, "%3.2f\n", ecuStatus->tempAir);
#endif
    return RET_OK;
}

int8_t VGEAR_answerEngineTemperature(ECU_STATUS* ecuStatus,char* response)
{
#ifdef NO_FLOAT_SPRINTF
    char tmpBuffer[12];
    dtostrf(temperature, 3, 2, tmpBuffer);
    sprintf(response, "%s\n", tmpBuffer);
#else
    sprintf(response, "%3.2f\n", ecuStatus->temperature);
#endif
    return RET_OK;
}

int8_t VGEAR_answerGear(ECU_STATUS* ecuStatus, char* response)
{
    sprintf(response, "%u\n", ecuStatus->currentGear);
    return RET_OK;
}

int8_t VGEAR_answerSpeed(ECU_STATUS* ecuStatus, char* response)
{
    sprintf(response, "%u\n", ecuStatus->currentSpeed);
    return RET_OK;
}

int8_t VGEAR_answerAll(ECU_STATUS* ecuStatus, char* response)
{
#ifdef NO_FLOAT_SPRINTF
    char tmpTempAir[12];
    char tmpTempEn[12];
    VGEAR_answerEngineTemperature(tmpTempEn);
    VGEAR_answerAirTemperature(tmpTempAir);
    removeNewLine(tmpTempEn);
    removeNewLine(tmpTempAir);
    sprintf(response, "%u %u %s %s %u %u\n", currentThrottle, currentRPM, tmpTempAir, tmpTempEn, currentGear, currentSpeed);
#else
    sprintf(response, "%u %u %3.2f %3.2f %u %u %u %u %u\n", ecuStatus->currentThrottle, ecuStatus->currentRPM, ecuStatus->tempAir, ecuStatus->temperature, ecuStatus->currentGear, ecuStatus->currentSpeed, ecuStatus->accelX, ecuStatus->accelY, ecuStatus->accelZ);
#endif
    return RET_OK;
}

int8_t VGEAR_answerStatus(ECU_STATUS* ecuStatus, char* response)
{
    sprintf(response, "%s\n", "E");
    return RET_OK;
}



int8_t VGEAR_parseCommand(ECU_STATUS* ecuStatus, const char* strCommand, char* response)
{
    char *pos = NULL;
    pos = strstr(strCommand, C_STATUS);
    if (NULL != pos)
    {
        VGEAR_answerStatus(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_ALL)))
    {
        VGEAR_answerAll(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_SPEED)))
    {
        VGEAR_answerSpeed(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_GEAR)))
    {
        VGEAR_answerGear(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_ETEMP)))
    {
        VGEAR_answerEngineTemperature(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_ATEMP)))
    {
        VGEAR_answerAirTemperature(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_RPM)))
    {
        VGEAR_answerRPM(ecuStatus, response);
        return RET_OK;
    }
    else if (NULL != (strstr(strCommand, C_THR)))
    {
        VGEAR_answerThrottle(ecuStatus, response);
        return RET_OK;
    }
    return E_ERROR_PARSING_REQUEST;
}


