/*
 * vgear.c
 *
 *  Created on: 17/4/2015
 *      Author: dzayas
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "type.h"
#include "vgear.h"
#include "vgear_request.h"

#include "printf.h"

#include "ret.h"

static int32_t isInitialized[UART_ID_LENGTH] = { 0,0,0,0 }; /* array to store if a UART is initialized */

static xSemaphoreHandle lstMutex[ UART_ID_LENGTH ] ; /* array of mutex */


int32_t _requestCommand(UART_ID uartId , const char* command, char* response )
{
    if ( 1 != isInitialized[uartId] )
    {
        return E_NON_INITIALIZED;
    }
    if( pdTRUE !=  xSemaphoreTake(lstMutex[uartId], portMAX_DELAY) )
    {
        return E_CANNOT_MUTEX;
    }
    int32_t ret = PrintF_print(uartId,command);
    if (RET_OK != ret )
    {
        xSemaphoreGive(  lstMutex[uartId] );
        return ret;
    }
    ret = PrintF_scan(uartId,response,3000);
    xSemaphoreGive(  lstMutex[uartId] );
    return ret;
}




int32_t VGEAR_connect( UART_ID uartId )
{
    int32_t ret = PrintF_start(uartId, C_VGEAR_BAUDRATE  );
    if( RET_OK == ret  )
    {
        isInitialized[uartId] = 1;
        lstMutex[uartId] = xSemaphoreCreateMutex();
    }
    return ret;
}

int32_t VGEAR_requestAll( UART_ID uartId, char* response )
{
    int32_t ret = _requestCommand( uartId, C_ALL, response );
    return ret;
}

int32_t VGEAR_requestSpeed( UART_ID uartId, char* response )
{
    int32_t ret = _requestCommand( uartId, C_SPEED, response );
    return ret;
}

int32_t VGEAR_requestStatus( UART_ID uartId, char* response )
{
    int32_t ret = _requestCommand( uartId, C_STATUS, response );
    return ret;
}

int32_t VGEAR_stringToECUStatus( const char* response, ECU_STATUS *sECUStatus )
{
//69 1200 180.32 200.55 4 45
    char strTmp[64];
    strcpy(strTmp, response);
    //printf("Response: %s\n", strTmp);
    int i = 0;
    i =		sscanf( strTmp, "%u %u %f %f %u %u",&sECUStatus->currentThrottle, &sECUStatus->currentRPM, &sECUStatus->tempAir
                    , &sECUStatus->temperature, &sECUStatus->currentGear, &sECUStatus->currentSpeed );
    //	printf("leido: %u %u %f %f %u %u\n",(sECUStatus->currentThrottle), (sECUStatus->currentRPM), sECUStatus->tempAir
    //	, sECUStatus->temperature, sECUStatus->currentGear, sECUStatus->currentSpeed);
    if ( 6  != i )
    {
        return E_ERROR_PARSING_ANSWER;
    }
    return RET_OK;
}

void VGEAR_ECUStatusToString( const char* output, ECU_STATUS *sECUStatus )
{
    sprintf( output, "%u %u %f %f %u %u %u %u %u\n", (sECUStatus->currentThrottle), (sECUStatus->currentRPM), sECUStatus->tempAir
             , sECUStatus->temperature, sECUStatus->currentGear, sECUStatus->currentSpeed, sECUStatus->accelX, sECUStatus->accelY, sECUStatus->accelZ );
}

